require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'
module StudioGame
	class Game
		attr_reader :title

		def initialize(title)
			@title = title
			@players = []
		end

		def load_players(from_file)
			File.readlines(from_file).each do |line|
				add_player(Player.from_csv(line))
			end
		end

		def save_high_scores(to_file='high_scores.txt')
			File.open(to_file, "w") do |file|
				file.puts "#{@title} High Scores:"
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end


		def high_score_entry(player)
			formatted_name = player.name.ljust(20, '.')
			"#{formatted_name} #{player.score}"
		end

		def add_player(player)
			@players << player
		end

		def play(rounds)
			puts "There are #{@players.count} players in #{@title}:"
			puts @players

			treasures = TreasureTrove::TREASURES
			puts "\nThere are #{treasures.count} treasures to be found"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points}"

			end

			1.upto(rounds) do |round|
				if block_given?
					break if yield
				end
				puts "Round #{round}: "
				@players.each do |player|
					GameTurn.take_turn(player)

					puts player
				end
			end
		end

		def print_stats
			strong_players, wimpy_players = @players.partition{|p|p.strong?}

			puts "\n#{@title} Statistics: "
			puts "\n#{strong_players.count} strong players:"
			strong_players.each do |sp|
				print_name_and_health(sp)
			end

			puts "\n#{wimpy_players.count} wimpy players:"
			wimpy_players.each do |wp|
				print_name_and_health(wp)
			end

			puts "\n#{@title} High Scores:"
			@players.sort.each do |player|
				puts high_score_entry(player)
			end

			@players.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "\nTotal points for all players: #{total_points}"
		end

		def total_points
				@players.reduce(0) do |sum, player|
					sum + player.points
				end
		end

		private

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end
	end
end
